@@ -46,11 +46,6 @@ void CPDF_RenderContext::AppendLayer(CPDF_PageObjectHolder* pObjectHolder,
   m_Layers.emplace_back(pObjectHolder, mtObject2Device);
 }
 
-void CPDF_RenderContext::Render(CFX_RenderDevice* pDevice,
-                                const CPDF_RenderOptions* pOptions) {
-  Render(pDevice, nullptr, pOptions, nullptr);
-}
-
 void CPDF_RenderContext::Render(CFX_RenderDevice* pDevice,
                                 const CPDF_PageObject* pStopObj,
                                 const CPDF_RenderOptions* pOptions,
