@@ -8,6 +8,7 @@
 
 #include <math.h>
 
+#include <iterator>
 #include <memory>
 #include <sstream>
 #include <utility>
@@ -23,10 +24,12 @@
 #include "core/fpdfapi/parser/cpdf_stream.h"
 #include "core/fpdfapi/parser/cpdf_string.h"
 #include "core/fpdfapi/parser/fpdf_parser_decode.h"
+#include "core/fpdfapi/parser/fpdf_parser_utility.h"
 #include "core/fpdfdoc/cpdf_bafontmap.h"
 #include "core/fpdfdoc/cpdf_formcontrol.h"
 #include "core/fpdfdoc/cpdf_icon.h"
 #include "core/fpdfdoc/cpvt_word.h"
+#include "core/fxcrt/fx_string_wrappers.h"
 #include "fpdfsdk/cpdfsdk_formfillenvironment.h"
 #include "fpdfsdk/cpdfsdk_interactiveform.h"
 #include "fpdfsdk/cpdfsdk_pageview.h"
@@ -34,7 +37,7 @@
 #include "fpdfsdk/pwl/cpwl_edit.h"
 #include "fpdfsdk/pwl/cpwl_edit_impl.h"
 #include "fpdfsdk/pwl/cpwl_wnd.h"
-#include "third_party/base/cxx17_backports.h"
+#include "third_party/base/numerics/safe_conversions.h"
 
 namespace {
 
@@ -85,7 +88,7 @@ const char kTextEndOperator[] = "ET";
 
 class AutoClosedCommand {
  public:
-  AutoClosedCommand(std::ostringstream* stream,
+  AutoClosedCommand(fxcrt::ostringstream* stream,
                     ByteString open,
                     ByteString close)
       : stream_(stream), close_(close) {
@@ -95,45 +98,56 @@ class AutoClosedCommand {
   virtual ~AutoClosedCommand() { *stream_ << close_ << "\n"; }
 
  private:
-  std::ostringstream* stream_;
+  UnownedPtr<fxcrt::ostringstream> const stream_;
   ByteString close_;
 };
 
 class AutoClosedQCommand final : public AutoClosedCommand {
  public:
-  explicit AutoClosedQCommand(std::ostringstream* stream)
+  explicit AutoClosedQCommand(fxcrt::ostringstream* stream)
       : AutoClosedCommand(stream, kStateSaveOperator, kStateRestoreOperator) {}
   ~AutoClosedQCommand() override = default;
 };
 
-ByteString GetColorAppStream(const CFX_Color& color,
-                             const bool& bFillOrStroke) {
-  std::ostringstream sColorStream;
-
+ByteString GetStrokeColorAppStream(const CFX_Color& color) {
+  fxcrt::ostringstream sColorStream;
   switch (color.nColorType) {
     case CFX_Color::Type::kTransparent:
       break;
     case CFX_Color::Type::kGray:
-      sColorStream << color.fColor1 << " "
-                   << (bFillOrStroke ? kSetGrayOperator
-                                     : kSetGrayStrokedOperator)
-                   << "\n";
+      sColorStream << color.fColor1 << " " << kSetGrayStrokedOperator << "\n";
       break;
     case CFX_Color::Type::kRGB:
       sColorStream << color.fColor1 << " " << color.fColor2 << " "
-                   << color.fColor3 << " "
-                   << (bFillOrStroke ? kSetRGBOperator : kSetRGBStrokedOperator)
-                   << "\n";
+                   << color.fColor3 << " " << kSetRGBStrokedOperator << "\n";
       break;
     case CFX_Color::Type::kCMYK:
       sColorStream << color.fColor1 << " " << color.fColor2 << " "
                    << color.fColor3 << " " << color.fColor4 << " "
-                   << (bFillOrStroke ? kSetCMYKOperator
-                                     : kSetCMKYStrokedOperator)
-                   << "\n";
+                   << kSetCMKYStrokedOperator << "\n";
       break;
   }
+  return ByteString(sColorStream);
+}
 
+ByteString GetFillColorAppStream(const CFX_Color& color) {
+  fxcrt::ostringstream sColorStream;
+  switch (color.nColorType) {
+    case CFX_Color::Type::kTransparent:
+      break;
+    case CFX_Color::Type::kGray:
+      sColorStream << color.fColor1 << " " << kSetGrayOperator << "\n";
+      break;
+    case CFX_Color::Type::kRGB:
+      sColorStream << color.fColor1 << " " << color.fColor2 << " "
+                   << color.fColor3 << " " << kSetRGBOperator << "\n";
+      break;
+    case CFX_Color::Type::kCMYK:
+      sColorStream << color.fColor1 << " " << color.fColor2 << " "
+                   << color.fColor3 << " " << color.fColor4 << " "
+                   << kSetCMYKOperator << "\n";
+      break;
+  }
   return ByteString(sColorStream);
 }
 
@@ -158,18 +172,18 @@ ByteString GetAP_Check(const CFX_FloatRect& crBBox) {
                           {CFX_PointF(0.40f, 0.60f), CFX_PointF(0.28f, 0.66f),
                            CFX_PointF(0.30f, 0.56f)}};
 
-  for (size_t i = 0; i < pdfium::size(pts); ++i) {
-    for (size_t j = 0; j < pdfium::size(pts[0]); ++j) {
+  for (size_t i = 0; i < std::size(pts); ++i) {
+    for (size_t j = 0; j < std::size(pts[0]); ++j) {
       pts[i][j].x = pts[i][j].x * fWidth + crBBox.left;
       pts[i][j].y *= pts[i][j].y * fHeight + crBBox.bottom;
     }
   }
 
-  std::ostringstream csAP;
+  fxcrt::ostringstream csAP;
   csAP << pts[0][0].x << " " << pts[0][0].y << " " << kMoveToOperator << "\n";
 
-  for (size_t i = 0; i < pdfium::size(pts); ++i) {
-    size_t nNext = i < pdfium::size(pts) - 1 ? i + 1 : 0;
+  for (size_t i = 0; i < std::size(pts); ++i) {
+    size_t nNext = i < std::size(pts) - 1 ? i + 1 : 0;
 
     float px1 = pts[i][1].x - pts[i][0].x;
     float py1 = pts[i][1].y - pts[i][0].y;
@@ -187,7 +201,7 @@ ByteString GetAP_Check(const CFX_FloatRect& crBBox) {
 }
 
 ByteString GetAP_Circle(const CFX_FloatRect& crBBox) {
-  std::ostringstream csAP;
+  fxcrt::ostringstream csAP;
 
   float fWidth = crBBox.Width();
   float fHeight = crBBox.Height();
@@ -231,7 +245,7 @@ ByteString GetAP_Circle(const CFX_FloatRect& crBBox) {
 }
 
 ByteString GetAP_Cross(const CFX_FloatRect& crBBox) {
-  std::ostringstream csAP;
+  fxcrt::ostringstream csAP;
 
   csAP << crBBox.left << " " << crBBox.top << " " << kMoveToOperator << "\n";
   csAP << crBBox.right << " " << crBBox.bottom << " " << kLineToOperator
@@ -243,7 +257,7 @@ ByteString GetAP_Cross(const CFX_FloatRect& crBBox) {
 }
 
 ByteString GetAP_Diamond(const CFX_FloatRect& crBBox) {
-  std::ostringstream csAP;
+  fxcrt::ostringstream csAP;
 
   float fWidth = crBBox.Width();
   float fHeight = crBBox.Height();
@@ -263,7 +277,7 @@ ByteString GetAP_Diamond(const CFX_FloatRect& crBBox) {
 }
 
 ByteString GetAP_Square(const CFX_FloatRect& crBBox) {
-  std::ostringstream csAP;
+  fxcrt::ostringstream csAP;
 
   csAP << crBBox.left << " " << crBBox.top << " " << kMoveToOperator << "\n";
   csAP << crBBox.right << " " << crBBox.top << " " << kLineToOperator << "\n";
@@ -276,7 +290,7 @@ ByteString GetAP_Square(const CFX_FloatRect& crBBox) {
 }
 
 ByteString GetAP_Star(const CFX_FloatRect& crBBox) {
-  std::ostringstream csAP;
+  fxcrt::ostringstream csAP;
 
   float fRadius = (crBBox.top - crBBox.bottom) / (1 + cosf(FXSYS_PI / 5.0f));
   CFX_PointF ptCenter = CFX_PointF((crBBox.left + crBBox.right) / 2.0f,
@@ -293,8 +307,8 @@ ByteString GetAP_Star(const CFX_FloatRect& crBBox) {
   csAP << points[0].x << " " << points[0].y << " " << kMoveToOperator << "\n";
 
   int next = 0;
-  for (size_t i = 0; i < pdfium::size(points); ++i) {
-    next = (next + 2) % pdfium::size(points);
+  for (size_t i = 0; i < std::size(points); ++i) {
+    next = (next + 2) % std::size(points);
     csAP << points[next].x << " " << points[next].y << " " << kLineToOperator
          << "\n";
   }
@@ -303,7 +317,7 @@ ByteString GetAP_Star(const CFX_FloatRect& crBBox) {
 }
 
 ByteString GetAP_HalfCircle(const CFX_FloatRect& crBBox, float fRotate) {
-  std::ostringstream csAP;
+  fxcrt::ostringstream csAP;
 
   float fWidth = crBBox.Width();
   float fHeight = crBBox.Height();
@@ -340,21 +354,21 @@ ByteString GetAP_HalfCircle(const CFX_FloatRect& crBBox, float fRotate) {
 
 ByteString GetAppStream_Check(const CFX_FloatRect& rcBBox,
                               const CFX_Color& crText) {
-  std::ostringstream sAP;
+  fxcrt::ostringstream sAP;
   {
     AutoClosedQCommand q(&sAP);
-    sAP << GetColorAppStream(crText, true) << GetAP_Check(rcBBox)
-        << kFillOperator << "\n";
+    sAP << GetFillColorAppStream(crText) << GetAP_Check(rcBBox) << kFillOperator
+        << "\n";
   }
   return ByteString(sAP);
 }
 
 ByteString GetAppStream_Circle(const CFX_FloatRect& rcBBox,
                                const CFX_Color& crText) {
-  std::ostringstream sAP;
+  fxcrt::ostringstream sAP;
   {
     AutoClosedQCommand q(&sAP);
-    sAP << GetColorAppStream(crText, true) << GetAP_Circle(rcBBox)
+    sAP << GetFillColorAppStream(crText) << GetAP_Circle(rcBBox)
         << kFillOperator << "\n";
   }
   return ByteString(sAP);
@@ -362,10 +376,10 @@ ByteString GetAppStream_Circle(const CFX_FloatRect& rcBBox,
 
 ByteString GetAppStream_Cross(const CFX_FloatRect& rcBBox,
                               const CFX_Color& crText) {
-  std::ostringstream sAP;
+  fxcrt::ostringstream sAP;
   {
     AutoClosedQCommand q(&sAP);
-    sAP << GetColorAppStream(crText, false) << GetAP_Cross(rcBBox)
+    sAP << GetStrokeColorAppStream(crText) << GetAP_Cross(rcBBox)
         << kStrokeOperator << "\n";
   }
   return ByteString(sAP);
@@ -373,11 +387,11 @@ ByteString GetAppStream_Cross(const CFX_FloatRect& rcBBox,
 
 ByteString GetAppStream_Diamond(const CFX_FloatRect& rcBBox,
                                 const CFX_Color& crText) {
-  std::ostringstream sAP;
+  fxcrt::ostringstream sAP;
   {
     AutoClosedQCommand q(&sAP);
     sAP << "1 " << kSetLineWidthOperator << "\n"
-        << GetColorAppStream(crText, true) << GetAP_Diamond(rcBBox)
+        << GetFillColorAppStream(crText) << GetAP_Diamond(rcBBox)
         << kFillOperator << "\n";
   }
   return ByteString(sAP);
@@ -385,10 +399,10 @@ ByteString GetAppStream_Diamond(const CFX_FloatRect& rcBBox,
 
 ByteString GetAppStream_Square(const CFX_FloatRect& rcBBox,
                                const CFX_Color& crText) {
-  std::ostringstream sAP;
+  fxcrt::ostringstream sAP;
   {
     AutoClosedQCommand q(&sAP);
-    sAP << GetColorAppStream(crText, true) << GetAP_Square(rcBBox)
+    sAP << GetFillColorAppStream(crText) << GetAP_Square(rcBBox)
         << kFillOperator << "\n";
   }
   return ByteString(sAP);
@@ -396,19 +410,19 @@ ByteString GetAppStream_Square(const CFX_FloatRect& rcBBox,
 
 ByteString GetAppStream_Star(const CFX_FloatRect& rcBBox,
                              const CFX_Color& crText) {
-  std::ostringstream sAP;
+  fxcrt::ostringstream sAP;
   {
     AutoClosedQCommand q(&sAP);
-    sAP << GetColorAppStream(crText, true) << GetAP_Star(rcBBox)
-        << kFillOperator << "\n";
+    sAP << GetFillColorAppStream(crText) << GetAP_Star(rcBBox) << kFillOperator
+        << "\n";
   }
   return ByteString(sAP);
 }
 
 ByteString GetCircleFillAppStream(const CFX_FloatRect& rect,
                                   const CFX_Color& color) {
-  std::ostringstream sAppStream;
-  ByteString sColor = GetColorAppStream(color, true);
+  fxcrt::ostringstream sAppStream;
+  ByteString sColor = GetFillColorAppStream(color);
   if (sColor.GetLength() > 0) {
     AutoClosedQCommand q(&sAppStream);
     sAppStream << sColor << GetAP_Circle(rect) << kFillOperator << "\n";
@@ -423,7 +437,7 @@ ByteString GetCircleBorderAppStream(const CFX_FloatRect& rect,
                                     const CFX_Color& crRightBottom,
                                     BorderStyle nStyle,
                                     const CPWL_Dash& dash) {
-  std::ostringstream sAppStream;
+  fxcrt::ostringstream sAppStream;
   ByteString sColor;
 
   if (fWidth > 0.0f) {
@@ -438,7 +452,7 @@ ByteString GetCircleBorderAppStream(const CFX_FloatRect& rect,
       default:
       case BorderStyle::kSolid:
       case BorderStyle::kUnderline: {
-        sColor = GetColorAppStream(color, false);
+        sColor = GetStrokeColorAppStream(color);
         if (sColor.GetLength() > 0) {
           AutoClosedQCommand q2(&sAppStream);
           sAppStream << fWidth << " " << kSetLineWidthOperator << "\n"
@@ -447,7 +461,7 @@ ByteString GetCircleBorderAppStream(const CFX_FloatRect& rect,
         }
       } break;
       case BorderStyle::kDash: {
-        sColor = GetColorAppStream(color, false);
+        sColor = GetStrokeColorAppStream(color);
         if (sColor.GetLength() > 0) {
           AutoClosedQCommand q2(&sAppStream);
           sAppStream << fWidth << " " << kSetLineWidthOperator << "\n"
@@ -458,23 +472,21 @@ ByteString GetCircleBorderAppStream(const CFX_FloatRect& rect,
         }
       } break;
       case BorderStyle::kBeveled: {
-        sColor = GetColorAppStream(color, false);
+        sColor = GetStrokeColorAppStream(color);
         if (sColor.GetLength() > 0) {
           AutoClosedQCommand q2(&sAppStream);
           sAppStream << fHalfWidth << " " << kSetLineWidthOperator << "\n"
                      << sColor << GetAP_Circle(rect) << " " << kStrokeOperator
                      << "\n";
         }
-
-        sColor = GetColorAppStream(crLeftTop, false);
+        sColor = GetStrokeColorAppStream(crLeftTop);
         if (sColor.GetLength() > 0) {
           AutoClosedQCommand q2(&sAppStream);
           sAppStream << fHalfWidth << " " << kSetLineWidthOperator << "\n"
                      << sColor << GetAP_HalfCircle(rect_by_75, FXSYS_PI / 4.0f)
                      << " " << kStrokeOperator << "\n";
         }
-
-        sColor = GetColorAppStream(crRightBottom, false);
+        sColor = GetStrokeColorAppStream(crRightBottom);
         if (sColor.GetLength() > 0) {
           AutoClosedQCommand q2(&sAppStream);
           sAppStream << fHalfWidth << " " << kSetLineWidthOperator << "\n"
@@ -484,23 +496,21 @@ ByteString GetCircleBorderAppStream(const CFX_FloatRect& rect,
         }
       } break;
       case BorderStyle::kInset: {
-        sColor = GetColorAppStream(color, false);
+        sColor = GetStrokeColorAppStream(color);
         if (sColor.GetLength() > 0) {
           AutoClosedQCommand q2(&sAppStream);
           sAppStream << fHalfWidth << " " << kSetLineWidthOperator << "\n"
                      << sColor << GetAP_Circle(rect) << " " << kStrokeOperator
                      << "\n";
         }
-
-        sColor = GetColorAppStream(crLeftTop, false);
+        sColor = GetStrokeColorAppStream(crLeftTop);
         if (sColor.GetLength() > 0) {
           AutoClosedQCommand q2(&sAppStream);
           sAppStream << fHalfWidth << " " << kSetLineWidthOperator << "\n"
                      << sColor << GetAP_HalfCircle(rect_by_75, FXSYS_PI / 4.0f)
                      << " " << kStrokeOperator << "\n";
         }
-
-        sColor = GetColorAppStream(crRightBottom, false);
+        sColor = GetStrokeColorAppStream(crRightBottom);
         if (sColor.GetLength() > 0) {
           AutoClosedQCommand q2(&sAppStream);
           sAppStream << fHalfWidth << " " << kSetLineWidthOperator << "\n"
@@ -574,16 +584,16 @@ ByteString GetFontSetString(IPVT_FontMap* pFontMap,
   if (sFontAlias.GetLength() <= 0 || fFontSize <= 0)
     return ByteString();
 
-  std::ostringstream sRet;
+  fxcrt::ostringstream sRet;
   sRet << "/" << sFontAlias << " " << fFontSize << " "
        << kSetTextFontAndSizeOperator << "\n";
   return ByteString(sRet);
 }
 
-ByteString GetWordRenderString(const ByteString& strWords) {
-  if (strWords.GetLength() > 0)
-    return PDF_EncodeString(strWords) + " " + kShowTextOperator + "\n";
-  return ByteString();
+ByteString GetWordRenderString(ByteStringView strWords) {
+  if (strWords.IsEmpty())
+    return ByteString();
+  return PDF_EncodeString(strWords) + " " + kShowTextOperator + "\n";
 }
 
 ByteString GetEditAppStream(CPWL_EditImpl* pEdit,
@@ -593,20 +603,20 @@ ByteString GetEditAppStream(CPWL_EditImpl* pEdit,
   CPWL_EditImpl::Iterator* pIterator = pEdit->GetIterator();
   pIterator->SetAt(0);
 
-  std::ostringstream sEditStream;
-  std::ostringstream sWords;
+  fxcrt::ostringstream sEditStream;
   int32_t nCurFontIndex = -1;
   CFX_PointF ptOld;
   CFX_PointF ptNew;
   CPVT_WordPlace oldplace;
+  ByteString sWords;
 
   while (pIterator->NextWord()) {
     CPVT_WordPlace place = pIterator->GetAt();
     if (bContinuous) {
       if (place.LineCmp(oldplace) != 0) {
-        if (sWords.tellp() > 0) {
-          sEditStream << GetWordRenderString(ByteString(sWords));
-          sWords.str("");
+        if (!sWords.IsEmpty()) {
+          sEditStream << GetWordRenderString(sWords.AsStringView());
+          sWords.clear();
         }
 
         CPVT_Word word;
@@ -631,18 +641,17 @@ ByteString GetEditAppStream(CPWL_EditImpl* pEdit,
       CPVT_Word word;
       if (pIterator->GetWord(word)) {
         if (word.nFontIndex != nCurFontIndex) {
-          if (sWords.tellp() > 0) {
-            sEditStream << GetWordRenderString(ByteString(sWords));
-            sWords.str("");
+          if (!sWords.IsEmpty()) {
+            sEditStream << GetWordRenderString(sWords.AsStringView());
+            sWords.clear();
           }
           sEditStream << GetFontSetString(pEdit->GetFontMap(), word.nFontIndex,
                                           word.fFontSize);
           nCurFontIndex = word.nFontIndex;
         }
 
-        sWords << pEdit->GetPDFWordString(nCurFontIndex, word.Word, SubWord);
+        sWords += pEdit->GetPDFWordString(nCurFontIndex, word.Word, SubWord);
       }
-
       oldplace = place;
     } else {
       CPVT_Word word;
@@ -655,25 +664,22 @@ ByteString GetEditAppStream(CPWL_EditImpl* pEdit,
                       << kMoveTextPositionOperator << "\n";
           ptOld = ptNew;
         }
-
         if (word.nFontIndex != nCurFontIndex) {
           sEditStream << GetFontSetString(pEdit->GetFontMap(), word.nFontIndex,
                                           word.fFontSize);
           nCurFontIndex = word.nFontIndex;
         }
-
         sEditStream << GetWordRenderString(
-            pEdit->GetPDFWordString(nCurFontIndex, word.Word, SubWord));
+            pEdit->GetPDFWordString(nCurFontIndex, word.Word, SubWord)
+                .AsStringView());
       }
     }
   }
 
-  if (sWords.tellp() > 0) {
-    sEditStream << GetWordRenderString(ByteString(sWords));
-    sWords.str("");
-  }
+  if (!sWords.IsEmpty())
+    sEditStream << GetWordRenderString(sWords.AsStringView());
 
-  std::ostringstream sAppStream;
+  fxcrt::ostringstream sAppStream;
   if (sEditStream.tellp() > 0) {
     sAppStream << sEditStream.str();
   }
@@ -686,15 +692,14 @@ ByteString GenerateIconAppStream(CPDF_IconFit& fit,
   if (rcIcon.IsEmpty() || !pIconStream)
     return ByteString();
 
-  auto pPDFIcon = std::make_unique<CPDF_Icon>(pIconStream);
-
-  CPWL_Wnd::CreateParams cp;
+  CPWL_Wnd::CreateParams cp(nullptr, nullptr, nullptr);
   cp.dwFlags = PWS_VISIBLE;
   auto pWnd = std::make_unique<CPWL_Wnd>(cp, nullptr);
   pWnd->Realize();
   if (!pWnd->Move(rcIcon, false, false))
     return ByteString();
 
+  auto pPDFIcon = std::make_unique<CPDF_Icon>(pIconStream);
   ByteString sAlias = pPDFIcon->GetImageAlias();
   if (sAlias.GetLength() <= 0)
     return ByteString();
@@ -705,7 +710,7 @@ ByteString GenerateIconAppStream(CPDF_IconFit& fit,
   const CFX_VectorF scale = fit.GetScale(image_size, rcPlate);
   const CFX_VectorF offset = fit.GetImageOffset(image_size, scale, rcPlate);
 
-  std::ostringstream str;
+  fxcrt::ostringstream str;
   {
     AutoClosedQCommand q(&str);
     str << rcPlate.left << " " << rcPlate.bottom << " "
@@ -890,7 +895,7 @@ ByteString GetPushButtonAppStream(const CFX_FloatRect& rcBBox,
       break;
   }
 
-  std::ostringstream sTemp;
+  fxcrt::ostringstream sTemp;
   sTemp << GenerateIconAppStream(IconFit, pIconStream, rcIcon);
 
   if (!rcLabel.IsEmpty()) {
@@ -900,14 +905,14 @@ ByteString GetPushButtonAppStream(const CFX_FloatRect& rcBBox,
         GetEditAppStream(pEdit.get(), CFX_PointF(0.0f, 0.0f), true, 0);
     if (sEdit.GetLength() > 0) {
       AutoClosedCommand bt(&sTemp, kTextBeginOperator, kTextEndOperator);
-      sTemp << GetColorAppStream(crText, true) << sEdit;
+      sTemp << GetFillColorAppStream(crText) << sEdit;
     }
   }
 
   if (sTemp.tellp() <= 0)
     return ByteString();
 
-  std::ostringstream sAppStream;
+  fxcrt::ostringstream sAppStream;
   {
     AutoClosedQCommand q(&sAppStream);
     sAppStream << rcBBox.left << " " << rcBBox.bottom << " "
@@ -927,7 +932,7 @@ ByteString GetBorderAppStreamInternal(const CFX_FloatRect& rect,
                                       const CFX_Color& crRightBottom,
                                       BorderStyle nStyle,
                                       const CPWL_Dash& dash) {
-  std::ostringstream sAppStream;
+  fxcrt::ostringstream sAppStream;
   ByteString sColor;
 
   float fLeft = rect.left;
@@ -942,7 +947,7 @@ ByteString GetBorderAppStreamInternal(const CFX_FloatRect& rect,
     switch (nStyle) {
       default:
       case BorderStyle::kSolid:
-        sColor = GetColorAppStream(color, true);
+        sColor = GetFillColorAppStream(color);
         if (sColor.GetLength() > 0) {
           sAppStream << sColor;
           sAppStream << fLeft << " " << fBottom << " " << fRight - fLeft << " "
@@ -955,7 +960,7 @@ ByteString GetBorderAppStreamInternal(const CFX_FloatRect& rect,
         }
         break;
       case BorderStyle::kDash:
-        sColor = GetColorAppStream(color, false);
+        sColor = GetStrokeColorAppStream(color);
         if (sColor.GetLength() > 0) {
           sAppStream << sColor;
           sAppStream << fWidth << " " << kSetLineWidthOperator << " ["
@@ -975,7 +980,7 @@ ByteString GetBorderAppStreamInternal(const CFX_FloatRect& rect,
         break;
       case BorderStyle::kBeveled:
       case BorderStyle::kInset:
-        sColor = GetColorAppStream(crLeftTop, true);
+        sColor = GetFillColorAppStream(crLeftTop);
         if (sColor.GetLength() > 0) {
           sAppStream << sColor;
           sAppStream << fLeft + fHalfWidth << " " << fBottom + fHalfWidth << " "
@@ -992,8 +997,7 @@ ByteString GetBorderAppStreamInternal(const CFX_FloatRect& rect,
                      << fBottom + fHalfWidth * 2 << " " << kLineToOperator
                      << " " << kFillOperator << "\n";
         }
-
-        sColor = GetColorAppStream(crRightBottom, true);
+        sColor = GetFillColorAppStream(crRightBottom);
         if (sColor.GetLength() > 0) {
           sAppStream << sColor;
           sAppStream << fRight - fHalfWidth << " " << fTop - fHalfWidth << " "
@@ -1011,8 +1015,7 @@ ByteString GetBorderAppStreamInternal(const CFX_FloatRect& rect,
           sAppStream << fRight - fHalfWidth * 2 << " " << fTop - fHalfWidth * 2
                      << " " << kLineToOperator << " " << kFillOperator << "\n";
         }
-
-        sColor = GetColorAppStream(color, true);
+        sColor = GetFillColorAppStream(color);
         if (sColor.GetLength() > 0) {
           sAppStream << sColor;
           sAppStream << fLeft << " " << fBottom << " " << fRight - fLeft << " "
@@ -1025,7 +1028,7 @@ ByteString GetBorderAppStreamInternal(const CFX_FloatRect& rect,
         }
         break;
       case BorderStyle::kUnderline:
-        sColor = GetColorAppStream(color, false);
+        sColor = GetStrokeColorAppStream(color);
         if (sColor.GetLength() > 0) {
           sAppStream << sColor;
           sAppStream << fWidth << " " << kSetLineWidthOperator << "\n";
@@ -1037,7 +1040,6 @@ ByteString GetBorderAppStreamInternal(const CFX_FloatRect& rect,
         break;
     }
   }
-
   return ByteString(sAppStream);
 }
 
@@ -1045,13 +1047,12 @@ ByteString GetDropButtonAppStream(const CFX_FloatRect& rcBBox) {
   if (rcBBox.IsEmpty())
     return ByteString();
 
-  std::ostringstream sAppStream;
+  fxcrt::ostringstream sAppStream;
   {
     AutoClosedQCommand q(&sAppStream);
-    sAppStream << GetColorAppStream(
+    sAppStream << GetFillColorAppStream(
                       CFX_Color(CFX_Color::Type::kRGB, 220.0f / 255.0f,
-                                220.0f / 255.0f, 220.0f / 255.0f),
-                      true)
+                                220.0f / 255.0f, 220.0f / 255.0f))
                << rcBBox.left << " " << rcBBox.bottom << " "
                << rcBBox.right - rcBBox.left << " "
                << rcBBox.top - rcBBox.bottom << " " << kAppendRectOperator
@@ -1088,8 +1089,8 @@ ByteString GetDropButtonAppStream(const CFX_FloatRect& rcBBox) {
 
 ByteString GetRectFillAppStream(const CFX_FloatRect& rect,
                                 const CFX_Color& color) {
-  std::ostringstream sAppStream;
-  ByteString sColor = GetColorAppStream(color, true);
+  fxcrt::ostringstream sAppStream;
+  ByteString sColor = GetFillColorAppStream(color);
   if (sColor.GetLength() > 0) {
     AutoClosedQCommand q(&sAppStream);
     sAppStream << sColor << rect.left << " " << rect.bottom << " "
@@ -1521,7 +1522,7 @@ void CPDFSDK_AppStream::SetAsRadioButton() {
 void CPDFSDK_AppStream::SetAsComboBox(absl::optional<WideString> sValue) {
   CPDF_FormControl* pControl = widget_->GetFormControl();
   CPDF_FormField* pField = pControl->GetField();
-  std::ostringstream sBody;
+  fxcrt::ostringstream sBody;
 
   CFX_FloatRect rcClient = widget_->GetClientRect();
   CFX_FloatRect rcButton = rcClient;
@@ -1580,7 +1581,7 @@ void CPDFSDK_AppStream::SetAsComboBox(absl::optional<WideString> sValue) {
 
     CFX_Color crText = widget_->GetTextPWLColor();
     AutoClosedCommand bt(&sBody, kTextBeginOperator, kTextEndOperator);
-    sBody << GetColorAppStream(crText, true) << sEdit;
+    sBody << GetFillColorAppStream(crText) << sEdit;
   }
 
   sBody << GetDropButtonAppStream(rcButton);
@@ -1593,7 +1594,7 @@ void CPDFSDK_AppStream::SetAsListBox() {
   CPDF_FormControl* pControl = widget_->GetFormControl();
   CPDF_FormField* pField = pControl->GetField();
   CFX_FloatRect rcClient = widget_->GetClientRect();
-  std::ostringstream sBody;
+  fxcrt::ostringstream sBody;
 
   // Font map must outlive |pEdit|.
   CPDF_BAFontMap font_map(widget_->GetPDFPage()->GetDocument(),
@@ -1608,7 +1609,7 @@ void CPDFSDK_AppStream::SetAsListBox() {
   pEdit->SetFontSize(FXSYS_IsFloatZero(fFontSize) ? 12.0f : fFontSize);
   pEdit->Initialize();
 
-  std::ostringstream sList;
+  fxcrt::ostringstream sList;
   float fy = rcClient.top;
 
   int32_t nTop = pField->GetTopVisibleIndex();
@@ -1635,22 +1636,21 @@ void CPDFSDK_AppStream::SetAsListBox() {
           CFX_FloatRect(rcClient.left, fy - fItemHeight, rcClient.right, fy);
       {
         AutoClosedQCommand q(&sList);
-        sList << GetColorAppStream(CFX_Color(CFX_Color::Type::kRGB, 0,
-                                             51.0f / 255.0f, 113.0f / 255.0f),
-                                   true)
+        sList << GetFillColorAppStream(CFX_Color(
+                     CFX_Color::Type::kRGB, 0, 51.0f / 255.0f, 113.0f / 255.0f))
               << rcItem.left << " " << rcItem.bottom << " " << rcItem.Width()
               << " " << rcItem.Height() << " " << kAppendRectOperator << " "
               << kFillOperator << "\n";
       }
 
       AutoClosedCommand bt(&sList, kTextBeginOperator, kTextEndOperator);
-      sList << GetColorAppStream(CFX_Color(CFX_Color::Type::kGray, 1), true)
+      sList << GetFillColorAppStream(CFX_Color(CFX_Color::Type::kGray, 1))
             << GetEditAppStream(pEdit.get(), CFX_PointF(0.0f, fy), true, 0);
     } else {
       CFX_Color crText = widget_->GetTextPWLColor();
 
       AutoClosedCommand bt(&sList, kTextBeginOperator, kTextEndOperator);
-      sList << GetColorAppStream(crText, true)
+      sList << GetFillColorAppStream(crText)
             << GetEditAppStream(pEdit.get(), CFX_PointF(0.0f, fy), true, 0);
     }
 
@@ -1677,8 +1677,8 @@ void CPDFSDK_AppStream::SetAsListBox() {
 void CPDFSDK_AppStream::SetAsTextField(absl::optional<WideString> sValue) {
   CPDF_FormControl* pControl = widget_->GetFormControl();
   CPDF_FormField* pField = pControl->GetField();
-  std::ostringstream sBody;
-  std::ostringstream sLines;
+  fxcrt::ostringstream sBody;
+  fxcrt::ostringstream sLines;
 
   // Font map must outlive |pEdit|.
   CPDF_BAFontMap font_map(widget_->GetPDFPage()->GetDocument(),
@@ -1725,7 +1725,7 @@ void CPDFSDK_AppStream::SetAsTextField(absl::optional<WideString> sValue) {
       }
     } else {
       if (sValue.has_value())
-        nMaxLen = sValue.value().GetLength();
+        nMaxLen = pdfium::base::checked_cast<int>(sValue.value().GetLength());
       pEdit->SetLimitChar(nMaxLen);
     }
   }
@@ -1760,19 +1760,19 @@ void CPDFSDK_AppStream::SetAsTextField(absl::optional<WideString> sValue) {
     CFX_Color crText = widget_->GetTextPWLColor();
 
     AutoClosedCommand bt(&sBody, kTextBeginOperator, kTextEndOperator);
-    sBody << GetColorAppStream(crText, true) << sEdit;
+    sBody << GetFillColorAppStream(crText) << sEdit;
   }
 
   if (bCharArray) {
     switch (widget_->GetBorderStyle()) {
       case BorderStyle::kSolid: {
         ByteString sColor =
-            GetColorAppStream(widget_->GetBorderPWLColor(), false);
+            GetStrokeColorAppStream(widget_->GetBorderPWLColor());
         if (sColor.GetLength() > 0) {
           AutoClosedQCommand q(&sLines);
           sLines << widget_->GetBorderWidth() << " " << kSetLineWidthOperator
                  << "\n"
-                 << GetColorAppStream(widget_->GetBorderPWLColor(), false)
+                 << GetStrokeColorAppStream(widget_->GetBorderPWLColor())
                  << " 2 " << kSetLineCapStyleOperator << " 0 "
                  << kSetLineJoinStyleOperator << "\n";
 
@@ -1790,14 +1790,14 @@ void CPDFSDK_AppStream::SetAsTextField(absl::optional<WideString> sValue) {
       }
       case BorderStyle::kDash: {
         ByteString sColor =
-            GetColorAppStream(widget_->GetBorderPWLColor(), false);
+            GetStrokeColorAppStream(widget_->GetBorderPWLColor());
         if (sColor.GetLength() > 0) {
           CPWL_Dash dsBorder = CPWL_Dash(3, 3, 0);
           AutoClosedQCommand q(&sLines);
           sLines << widget_->GetBorderWidth() << " " << kSetLineWidthOperator
                  << "\n"
-                 << GetColorAppStream(widget_->GetBorderPWLColor(), false)
-                 << "[" << dsBorder.nDash << " " << dsBorder.nGap << "] "
+                 << GetStrokeColorAppStream(widget_->GetBorderPWLColor()) << "["
+                 << dsBorder.nDash << " " << dsBorder.nGap << "] "
                  << dsBorder.nPhase << " " << kSetDashOperator << "\n";
 
           for (int32_t i = 1; i < nMaxLen; ++i) {
@@ -1833,10 +1833,7 @@ void CPDFSDK_AppStream::AddImage(const ByteString& sAPType,
   if (pImageDict)
     sImageAlias = pImageDict->GetStringFor("Name");
 
-  CPDF_Dictionary* pStreamResList = pStreamDict->GetDictFor("Resources");
-  if (!pStreamResList)
-    pStreamResList = pStreamDict->SetNewFor<CPDF_Dictionary>("Resources");
-
+  CPDF_Dictionary* pStreamResList = GetOrCreateDict(pStreamDict, "Resources");
   CPDF_Dictionary* pXObject =
       pStreamResList->SetNewFor<CPDF_Dictionary>("XObject");
   pXObject->SetNewFor<CPDF_Reference>(sImageAlias,
@@ -1847,24 +1844,27 @@ void CPDFSDK_AppStream::AddImage(const ByteString& sAPType,
 void CPDFSDK_AppStream::Write(const ByteString& sAPType,
                               const ByteString& sContents,
                               const ByteString& sAPState) {
-  CPDF_Stream* pStream = nullptr;
-  CPDF_Dictionary* pParentDict = nullptr;
+  CPDF_Dictionary* pParentDict;
+  ByteString key;
   if (sAPState.IsEmpty()) {
     pParentDict = dict_.Get();
-    pStream = dict_->GetStreamFor(sAPType);
+    key = sAPType;
   } else {
-    CPDF_Dictionary* pAPTypeDict = dict_->GetDictFor(sAPType);
-    if (!pAPTypeDict)
-      pAPTypeDict = dict_->SetNewFor<CPDF_Dictionary>(sAPType);
-
-    pParentDict = pAPTypeDict;
-    pStream = pAPTypeDict->GetStreamFor(sAPState);
+    pParentDict = GetOrCreateDict(dict_.Get(), sAPType);
+    key = sAPState;
   }
 
-  if (!pStream) {
-    CPDF_Document* doc = widget_->GetPageView()->GetPDFDocument();
-    pStream = doc->NewIndirect<CPDF_Stream>();
-    pParentDict->SetNewFor<CPDF_Reference>(sAPType, doc, pStream->GetObjNum());
+  RetainPtr<CPDF_Dictionary> pOrigStreamDict;
+
+  // If `pStream` is created by CreateModifiedAPStream(), then it is safe to
+  // edit, as it is not shared.
+  CPDF_Stream* pStream = pParentDict->GetStreamFor(key);
+  CPDF_Document* doc = widget_->GetPageView()->GetPDFDocument();
+  if (!doc->IsModifiedAPStream(pStream)) {
+    if (pStream)
+      pOrigStreamDict = pStream->GetDict();
+    pStream = doc->CreateModifiedAPStream();
+    pParentDict->SetNewFor<CPDF_Reference>(key, doc, pStream->GetObjNum());
   }
 
   CPDF_Dictionary* pStreamDict = pStream->GetDict();
@@ -1875,6 +1875,13 @@ void CPDFSDK_AppStream::Write(const ByteString& sAPType,
     pStreamDict->SetNewFor<CPDF_Name>("Type", "XObject");
     pStreamDict->SetNewFor<CPDF_Name>("Subtype", "Form");
     pStreamDict->SetNewFor<CPDF_Number>("FormType", 1);
+
+    if (pOrigStreamDict) {
+      CPDF_Dictionary* pResources = pOrigStreamDict->GetDictFor("Resources");
+      if (pResources)
+        pStreamDict->SetFor("Resources", pResources->Clone());
+    }
+
     pStream->InitStream({}, std::move(pNewDict));
   }
   pStreamDict->SetMatrixFor("Matrix", widget_->GetMatrix());
@@ -1882,7 +1889,7 @@ void CPDFSDK_AppStream::Write(const ByteString& sAPType,
   pStream->SetDataAndRemoveFilter(sContents.raw_span());
 }
 
-void CPDFSDK_AppStream::Remove(const ByteString& sAPType) {
+void CPDFSDK_AppStream::Remove(ByteStringView sAPType) {
   dict_->RemoveFor(sAPType);
 }
 
