@@ -26,6 +26,7 @@
 #include "fxbarcode/qrcode/BC_QRCoder.h"
 #include "fxbarcode/qrcode/BC_QRCoderErrorCorrectionLevel.h"
 #include "fxbarcode/qrcode/BC_QRCoderMode.h"
+#include "third_party/base/numerics/safe_conversions.h"
 
 CBC_QRCoder::CBC_QRCoder() = default;
 
@@ -55,10 +56,6 @@ int32_t CBC_QRCoder::GetNumDataBytes() const {
   return m_numDataBytes;
 }
 
-int32_t CBC_QRCoder::GetNumECBytes() const {
-  return m_numECBytes;
-}
-
 int32_t CBC_QRCoder::GetNumRSBlocks() const {
   return m_numRSBlocks;
 }
@@ -73,7 +70,8 @@ bool CBC_QRCoder::IsValid() const {
          m_numECBytes != -1 && m_numRSBlocks != -1 &&
          IsValidMaskPattern(m_maskPattern) &&
          m_numTotalBytes == m_numDataBytes + m_numECBytes && m_matrix &&
-         m_matrixWidth == m_matrix->GetWidth() &&
+         m_matrixWidth ==
+             pdfium::base::checked_cast<int32_t>(m_matrix->GetWidth()) &&
          m_matrix->GetWidth() == m_matrix->GetHeight();
 }
 
