@@ -29,7 +29,6 @@
 #include "fxbarcode/qrcode/BC_QRCoderBitVector.h"
 #include "fxbarcode/qrcode/BC_QRCoderECBlocksData.h"
 #include "fxbarcode/qrcode/BC_QRCoderErrorCorrectionLevel.h"
-#include "third_party/base/ptr_util.h"
 
 namespace {
 
@@ -40,10 +39,10 @@ std::vector<std::unique_ptr<CBC_QRCoderVersion>>* g_VERSION = nullptr;
 CBC_QRCoderVersion::CBC_QRCoderVersion(int32_t versionNumber,
                                        const CBC_QRCoderECBlockData data[4])
     : m_versionNumber(versionNumber) {
-  m_ecBlocksArray[0] = pdfium::MakeUnique<CBC_QRCoderECBlocks>(data[0]);
-  m_ecBlocksArray[1] = pdfium::MakeUnique<CBC_QRCoderECBlocks>(data[1]);
-  m_ecBlocksArray[2] = pdfium::MakeUnique<CBC_QRCoderECBlocks>(data[2]);
-  m_ecBlocksArray[3] = pdfium::MakeUnique<CBC_QRCoderECBlocks>(data[3]);
+  m_ecBlocksArray[0] = std::make_unique<CBC_QRCoderECBlocks>(data[0]);
+  m_ecBlocksArray[1] = std::make_unique<CBC_QRCoderECBlocks>(data[1]);
+  m_ecBlocksArray[2] = std::make_unique<CBC_QRCoderECBlocks>(data[2]);
+  m_ecBlocksArray[3] = std::make_unique<CBC_QRCoderECBlocks>(data[3]);
   m_totalCodeWords = m_ecBlocksArray[0]->GetTotalDataCodeWords();
 }
 
@@ -66,7 +65,7 @@ const CBC_QRCoderVersion* CBC_QRCoderVersion::GetVersionForNumber(
   if (g_VERSION->empty()) {
     for (int i = 0; i < kMaxVersion; ++i) {
       g_VERSION->push_back(
-          pdfium::MakeUnique<CBC_QRCoderVersion>(i + 1, g_ECBData[i]));
+          std::make_unique<CBC_QRCoderVersion>(i + 1, fxbarcode::kECBData[i]));
     }
   }
   if (versionNumber < 1 || versionNumber > kMaxVersion)
