@@ -11,6 +11,8 @@
 #include "include/core/SkTypes.h"
 #include "include/private/SkChecksum.h"
 #include "include/private/SkTemplates.h"
+
+#include <initializer_list>
 #include <new>
 #include <utility>
 
@@ -96,8 +98,8 @@ public:
             if (s.empty()) {
                 return nullptr;
             }
-            if (hash == s.hash && key == Traits::GetKey(s.val)) {
-                return &s.val;
+            if (hash == s.hash && key == Traits::GetKey(*s)) {
+                return &*s;
             }
             index = this->next(index);
         }
@@ -122,8 +124,8 @@ public:
         int index = hash & (fCapacity-1);
         for (int n = 0; n < fCapacity; n++) {
             Slot& s = fSlots[index];
-            SkASSERT(!s.empty());
-            if (hash == s.hash && key == Traits::GetKey(s.val)) {
+            SkASSERT(s.has_value());
+            if (hash == s.hash && key == Traits::GetKey(*s)) {
                this->removeSlot(index);
                if (4 * fCount <= fCapacity && fCapacity > 4) {
                    this->resize(fCapacity / 2);
@@ -134,12 +136,33 @@ public:
         }
     }
 
+    // Hash tables will automatically resize themselves when set() and remove() are called, but
+    // resize() can be called to manually grow capacity before a bulk insertion.
+    void resize(int capacity) {
+        SkASSERT(capacity >= fCount);
+        int oldCapacity = fCapacity;
+        SkDEBUGCODE(int oldCount = fCount);
+
+        fCount = 0;
+        fCapacity = capacity;
+        SkAutoTArray<Slot> oldSlots = std::move(fSlots);
+        fSlots = SkAutoTArray<Slot>(capacity);
+
+        for (int i = 0; i < oldCapacity; i++) {
+            Slot& s = oldSlots[i];
+            if (s.has_value()) {
+                this->uncheckedSet(*std::move(s));
+            }
+        }
+        SkASSERT(fCount == oldCount);
+    }
+
     // Call fn on every entry in the table.  You may mutate the entries, but be very careful.
     template <typename Fn>  // f(T*)
     void foreach(Fn&& fn) {
         for (int i = 0; i < fCapacity; i++) {
-            if (!fSlots[i].empty()) {
-                fn(&fSlots[i].val);
+            if (fSlots[i].has_value()) {
+                fn(&*fSlots[i]);
             }
         }
     }
@@ -148,8 +171,8 @@ public:
     template <typename Fn>  // f(T) or f(const T&)
     void foreach(Fn&& fn) const {
         for (int i = 0; i < fCapacity; i++) {
-            if (!fSlots[i].empty()) {
-                fn(fSlots[i].val);
+            if (fSlots[i].has_value()) {
+                fn(*fSlots[i]);
             }
         }
     }
@@ -210,7 +233,7 @@ private:
     // Finds the first non-empty slot for an iterator.
     int firstPopulatedSlot() const {
         for (int i = 0; i < fCapacity; i++) {
-            if (!fSlots[i].empty()) {
+            if (fSlots[i].has_value()) {
                 return i;
             }
         }
@@ -220,7 +243,7 @@ private:
     // Increments an iterator's slot.
     int nextPopulatedSlot(int currentSlot) const {
         for (int i = currentSlot + 1; i < fCapacity; i++) {
-            if (!fSlots[i].empty()) {
+            if (fSlots[i].has_value()) {
                 return i;
             }
         }
@@ -229,28 +252,28 @@ private:
 
     // Reads from an iterator's slot.
     const T* slot(int i) const {
-        SkASSERT(!fSlots[i].empty());
-        return &fSlots[i].val;
+        SkASSERT(fSlots[i].has_value());
+        return &*fSlots[i];
     }
 
     T* uncheckedSet(T&& val) {
         const K& key = Traits::GetKey(val);
+        SkASSERT(key == key);
         uint32_t hash = Hash(key);
         int index = hash & (fCapacity-1);
         for (int n = 0; n < fCapacity; n++) {
             Slot& s = fSlots[index];
             if (s.empty()) {
                 // New entry.
-                s.val  = std::move(val);
-                s.hash = hash;
+                s.emplace(std::move(val), hash);
                 fCount++;
-                return &s.val;
+                return &*s;
             }
-            if (hash == s.hash && key == Traits::GetKey(s.val)) {
+            if (hash == s.hash && key == Traits::GetKey(*s)) {
                 // Overwrite previous entry.
                 // Note: this triggers extra copies when adding the same value repeatedly.
-                s.val = std::move(val);
-                return &s.val;
+                s.emplace(std::move(val), hash);
+                return &*s;
             }
 
             index = this->next(index);
@@ -259,24 +282,6 @@ private:
         return nullptr;
     }
 
-    void resize(int capacity) {
-        int oldCapacity = fCapacity;
-        SkDEBUGCODE(int oldCount = fCount);
-
-        fCount = 0;
-        fCapacity = capacity;
-        SkAutoTArray<Slot> oldSlots = std::move(fSlots);
-        fSlots = SkAutoTArray<Slot>(capacity);
-
-        for (int i = 0; i < oldCapacity; i++) {
-            Slot& s = oldSlots[i];
-            if (!s.empty()) {
-                this->uncheckedSet(std::move(s.val));
-            }
-        }
-        SkASSERT(fCount == oldCount);
-    }
-
     void removeSlot(int index) {
         fCount--;
 
@@ -296,7 +301,7 @@ private:
                 Slot& s = fSlots[index];
                 if (s.empty()) {
                     // We're done shuffling elements around.  Clear the last empty slot.
-                    emptySlot = Slot();
+                    emptySlot.reset();
                     return;
                 }
                 originalIndex = s.hash & (fCapacity - 1);
@@ -322,12 +327,85 @@ private:
 
     struct Slot {
         Slot() = default;
-        Slot(T&& v, uint32_t h) : val(std::move(v)), hash(h) {}
+        ~Slot() { this->reset(); }
 
-        bool empty() const { return this->hash == 0; }
+        Slot(const Slot& that) { *this = that; }
+        Slot& operator=(const Slot& that) {
+            if (this == &that) {
+                return *this;
+            }
+            if (hash) {
+                if (that.hash) {
+                    val.storage = that.val.storage;
+                    hash = that.hash;
+                } else {
+                    this->reset();
+                }
+            } else {
+                if (that.hash) {
+                    new (&val.storage) T(that.val.storage);
+                    hash = that.hash;
+                } else {
+                    // do nothing, no value on either side
+                }
+            }
+            return *this;
+        }
+
+        Slot(Slot&& that) { *this = std::move(that); }
+        Slot& operator=(Slot&& that) {
+            if (this == &that) {
+                return *this;
+            }
+            if (hash) {
+                if (that.hash) {
+                    val.storage = std::move(that.val.storage);
+                    hash = that.hash;
+                } else {
+                    this->reset();
+                }
+            } else {
+                if (that.hash) {
+                    new (&val.storage) T(std::move(that.val.storage));
+                    hash = that.hash;
+                } else {
+                    // do nothing, no value on either side
+                }
+            }
+            return *this;
+        }
+
+        T& operator*() & { return val.storage; }
+        const T& operator*() const& { return val.storage; }
+        T&& operator*() && { return std::move(val.storage); }
+        const T&& operator*() const&& { return std::move(val.storage); }
+
+        Slot& emplace(T&& v, uint32_t h) {
+            this->reset();
+            new (&val.storage) T(std::move(v));
+            hash = h;
+            return *this;
+        }
+
+        bool has_value() const { return hash != 0; }
+        explicit operator bool() const { return this->has_value(); }
+        bool empty() const { return !this->has_value(); }
+
+        void reset() {
+            if (hash) {
+                val.storage.~T();
+                hash = 0;
+            }
+        }
 
-        T        val{};
         uint32_t hash = 0;
+
+    private:
+        union Storage {
+            T storage;
+            Storage() {}
+            ~Storage() {}
+        } val;
     };
 
     int fCount    = 0,
@@ -340,12 +418,38 @@ private:
 template <typename K, typename V, typename HashK = SkGoodHash>
 class SkTHashMap {
 public:
+    // Allow default construction and assignment.
+    SkTHashMap() = default;
+
+    SkTHashMap(SkTHashMap<K, V, HashK>&& that) = default;
+    SkTHashMap(const SkTHashMap<K, V, HashK>& that) = default;
+
+    SkTHashMap<K, V, HashK>& operator=(SkTHashMap<K, V, HashK>&& that) = default;
+    SkTHashMap<K, V, HashK>& operator=(const SkTHashMap<K, V, HashK>& that) = default;
+
+    // Construct with an initializer list of key-value pairs.
+    struct Pair : public std::pair<K, V> {
+        using std::pair<K, V>::pair;
+        static const K& GetKey(const Pair& p) { return p.first; }
+        static auto Hash(const K& key) { return HashK()(key); }
+    };
+
+    SkTHashMap(std::initializer_list<Pair> pairs) {
+        fTable.resize(pairs.size() * 5 / 3);
+        for (const Pair& p : pairs) {
+            fTable.set(p);
+        }
+    }
+
     // Clear the map.
     void reset() { fTable.reset(); }
 
     // How many key/value pairs are in the table?
     int count() const { return fTable.count(); }
 
+    // Is empty?
+    bool empty() const { return fTable.count() == 0; }
+
     // Approximately how many bytes of memory do we use beyond sizeof(*this)?
     size_t approxBytesUsed() const { return fTable.approxBytesUsed(); }
 
@@ -393,12 +497,6 @@ public:
     }
 
     // Dereferencing an iterator gives back a key-value pair, suitable for structured binding.
-    struct Pair : public std::pair<K, V> {
-        using std::pair<K, V>::pair;
-        static const K& GetKey(const Pair& p) { return p.first; }
-        static auto Hash(const K& key) { return HashK()(key); }
-    };
-
     using Iter = typename SkTHashTable<Pair, K>::template Iter<std::pair<K, V>>;
 
     Iter begin() const {
@@ -417,6 +515,23 @@ private:
 template <typename T, typename HashT = SkGoodHash>
 class SkTHashSet {
 public:
+    // Allow default construction and assignment.
+    SkTHashSet() = default;
+
+    SkTHashSet(SkTHashSet<T, HashT>&& that) = default;
+    SkTHashSet(const SkTHashSet<T, HashT>& that) = default;
+
+    SkTHashSet<T, HashT>& operator=(SkTHashSet<T, HashT>&& that) = default;
+    SkTHashSet<T, HashT>& operator=(const SkTHashSet<T, HashT>& that) = default;
+
+    // Construct with an initializer list of Ts.
+    SkTHashSet(std::initializer_list<T> vals) {
+        fTable.resize(vals.size() * 5 / 3);
+        for (const T& val : vals) {
+            fTable.set(val);
+        }
+    }
+
     // Clear the set.
     void reset() { fTable.reset(); }
 
