@@ -5,9 +5,6 @@
 #if !defined __XML_WSOCKCOMPAT_H__
 #define __XML_WSOCKCOMPAT_H__
 
-#ifdef _WIN32_WCE
-#include <winsock.h>
-#else
 #include <errno.h>
 #include <winsock2.h>
 
@@ -30,7 +27,6 @@
 #include <wspiapi.h>
 #define HAVE_GETADDRINFO
 #endif
-#endif
 
 #undef XML_SOCKLEN_T
 #define XML_SOCKLEN_T int
