@@ -37,56 +37,10 @@ using librevenge::RVNG_POINT;
 using namespace std::placeholders;
 
 using std::make_shared;
-using std::memcmp;
 using std::shared_ptr;
-using std::string;
 
 namespace
 {
-
-const unsigned char SIGNATURE_PDF[] = { '%', 'P', 'D', 'F' };
-const unsigned char SIGNATURE_PNG[] = { 0x89, 'P', 'N', 'G', 0x0d, 0x0a, 0x1a, 0x0a };
-const unsigned char SIGNATURE_JPEG[] = { 0xff, 0xd8 };
-const unsigned char SIGNATURE_QUICKTIME[] = { 'm', 'o', 'o', 'v' };
-const unsigned char SIGNATURE_TIFF_1[] = { 0x49, 0x49, 0x2a, 0x00 };
-const unsigned char SIGNATURE_TIFF_2[] = { 0x4d, 0x4d, 0x00, 0x2a };
-
-string detectMimetype(const RVNGInputStreamPtr_t &stream)
-{
-  stream->seek(0, librevenge::RVNG_SEEK_SET);
-
-  unsigned long numBytesRead = 0;
-  const unsigned char *const sig = stream->read(8, numBytesRead);
-
-  if (8 != numBytesRead)
-    // looks like the binary is broken anyway: just bail out
-    return string();
-
-  if (0 == memcmp(sig, SIGNATURE_PNG, ETONYEK_NUM_ELEMENTS(SIGNATURE_PNG)))
-    return string("image/png");
-
-  if (0 == memcmp(sig, SIGNATURE_PDF, ETONYEK_NUM_ELEMENTS(SIGNATURE_PDF)))
-    return string("application/pdf");
-
-  if ((0 == memcmp(sig, SIGNATURE_TIFF_1, ETONYEK_NUM_ELEMENTS(SIGNATURE_TIFF_1)))
-      || (0 == memcmp(sig, SIGNATURE_TIFF_2, ETONYEK_NUM_ELEMENTS(SIGNATURE_TIFF_2))))
-    return string("image/tiff");
-
-  if (0 == memcmp(sig + 4, SIGNATURE_QUICKTIME, ETONYEK_NUM_ELEMENTS(SIGNATURE_QUICKTIME)))
-    return string("video/quicktime");
-
-  if (0 == memcmp(sig, SIGNATURE_JPEG, ETONYEK_NUM_ELEMENTS(SIGNATURE_JPEG)))
-    return string("image/jpeg");
-
-  static bool first=true;
-  if (first)
-  {
-    ETONYEK_DEBUG_MSG(("detectMimetype[IWORKCollector.cpp]: can not detect some stream types\n"));
-    first=false;
-  }
-  return string();
-}
-
 librevenge::RVNGPropertyList makePoint(const double x, const double y)
 {
   librevenge::RVNGPropertyList props;
@@ -229,8 +183,8 @@ private:
 IWORKCollector::Level::Level()
   : m_geometry()
   , m_graphicStyle()
-  , m_trafo()
-  , m_previousTrafo()
+  , m_trafo(1)
+  , m_previousTrafo(1)
 {
 }
 
@@ -259,6 +213,7 @@ IWORKCollector::IWORKCollector(IWORKDocumentInterface *const document)
   , m_metadata()
   , m_accumulateTransform(true)
   , m_groupLevel(0)
+  , m_groupOpenLevel(0)
 {
 }
 
@@ -266,7 +221,7 @@ IWORKCollector::~IWORKCollector()
 {
   assert(m_levelStack.empty());
   assert(m_stylesheetStack.empty());
-  assert(0 == m_groupLevel);
+  assert(0 == m_groupLevel && 0 == m_groupOpenLevel);
 
   assert(!m_currentPath);
   assert(!m_currentText);
@@ -379,11 +334,11 @@ void IWORKCollector::collectLine(const IWORKLinePtr_t &line)
   drawLine(line);
 }
 
-void IWORKCollector::collectShape(const boost::optional<int> &order, bool locked)
+void IWORKCollector::collectShape(const boost::optional<int> &order, const boost::optional<unsigned> &resizeFlags, bool locked)
 {
   if (bool(m_recorder))
   {
-    m_recorder->collectShape(order, locked);
+    m_recorder->collectShape(order, resizeFlags, locked);
     return;
   }
 
@@ -408,6 +363,7 @@ void IWORKCollector::collectShape(const boost::optional<int> &order, bool locked
   }
 
   shape->m_order = order;
+  shape->m_resizeFlags=resizeFlags;
   shape->m_locked = locked;
   shape->m_style = m_levelStack.top().m_graphicStyle;
   m_levelStack.top().m_graphicStyle.reset();
@@ -558,16 +514,21 @@ void IWORKCollector::collectText(const std::shared_ptr<IWORKText> &text)
   m_currentText = text;
 }
 
-void IWORKCollector::startDocument()
+void IWORKCollector::collectStickyNote()
 {
-  m_document->startDocument(librevenge::RVNGPropertyList());
+  ETONYEK_DEBUG_MSG(("IWORKCollector::collectStickyNote: not implemented\n"));
+}
+
+void IWORKCollector::startDocument(const librevenge::RVNGPropertyList &props)
+{
+  m_document->startDocument(props);
 }
 
 void IWORKCollector::endDocument()
 {
   assert(m_levelStack.empty());
   assert(m_pathStack.empty());
-  assert(0 == m_groupLevel);
+  assert(0 == m_groupLevel && 0 == m_groupOpenLevel);
 
   assert(!m_currentPath);
   assert(!m_currentText);
@@ -609,6 +570,7 @@ void IWORKCollector::openGroup()
 
   assert(m_groupLevel > 0);
   m_outputManager.getCurrent().addOpenGroup(librevenge::RVNGPropertyList());
+  ++m_groupOpenLevel;
 }
 
 void IWORKCollector::closeGroup()
@@ -618,8 +580,9 @@ void IWORKCollector::closeGroup()
     m_recorder->closeGroup();
     return;
   }
-  assert(m_groupLevel > 0);
+  assert(m_groupLevel > 0 && m_groupOpenLevel > 0);
   m_outputManager.getCurrent().addCloseGroup();
+  --m_groupOpenLevel;
 }
 
 std::shared_ptr<IWORKTable> IWORKCollector::createTable(const IWORKTableNameMapPtr_t &tableNameMap, const IWORKLanguageManager &langManager) const
@@ -640,7 +603,7 @@ void IWORKCollector::startLevel()
     return;
   }
 
-  glm::dmat3 currentTrafo, prevTrafo;
+  glm::dmat3 currentTrafo(1), prevTrafo(1);
   if (!m_levelStack.empty())
   {
     currentTrafo = m_levelStack.top().m_trafo;
@@ -929,8 +892,7 @@ void IWORKCollector::fillGraphicProps(const IWORKStylePtr_t style, librevenge::R
   if (style->has<Shadow>())
   {
     const IWORKShadow &shadow = style->get<Shadow>();
-
-    props.insert("draw:shadow", "visible");
+    props.insert("draw:shadow", shadow.m_visible ? "visible" : "hidden");
     props.insert("draw:shadow-color", makeColor(shadow.m_color));
     props.insert("draw:shadow-opacity", shadow.m_opacity, RVNG_PERCENT);
     const double angle = deg2rad(shadow.m_angle);
@@ -981,6 +943,26 @@ void IWORKCollector::fillLayoutProps(const IWORKStylePtr_t layoutStyle, libreven
   }
 }
 
+void IWORKCollector::fillTextAutoSizeProps(const boost::optional<unsigned> &resizeFlags, const IWORKGeometryPtr_t &boundingBox, librevenge::RVNGPropertyList &props)
+{
+  if (!resizeFlags) return;
+  unsigned const flags=get(resizeFlags)&3;
+  // checkme: set "draw:auto-grow-width" to true seems to create too many problems
+  if ((flags&1)==1 && boundingBox && boundingBox->m_naturalSize.m_width>0)
+    props.insert("draw:auto-grow-width",false);
+  if ((flags&2)==0)
+    props.insert("draw:auto-grow-height",true);
+  else if (boundingBox && boundingBox->m_naturalSize.m_height>0)
+  {
+    props.insert("draw:auto-grow-height",false);
+    // if ((flags&1)==0) props.insert("draw:auto-grow-width",true); checkme
+
+    // the following seems only use in Impress
+    props.insert("draw:fit-to-size",true);
+    props.insert("style:shrink-to-fit",true);
+  }
+}
+
 void IWORKCollector::fillWrapProps(const IWORKStylePtr_t style, librevenge::RVNGPropertyList &props,
                                    const boost::optional<int> &order)
 {
@@ -1110,6 +1092,7 @@ void IWORKCollector::drawLine(const IWORKLinePtr_t &line)
 
   librevenge::RVNGPropertyList points;
   points.insert("svg:points", vertices);
+  fillShapeProperties(points);
 
   elements.addDrawPolyline(points);
 }
@@ -1125,7 +1108,7 @@ void IWORKCollector::drawMedia(const IWORKMediaPtr_t &media)
     const glm::dmat3 trafo = m_levelStack.top().m_trafo;
     const RVNGInputStreamPtr_t input = media->m_content->m_data->m_stream;
 
-    string mimetype(media->m_content->m_data->m_mimeType);
+    std::string mimetype(media->m_content->m_data->m_mimeType);
     if (mimetype.empty())
       mimetype = detectMimetype(input);
     if (!mimetype.empty())
@@ -1223,6 +1206,7 @@ void IWORKCollector::drawShape(const IWORKShapePtr_t &shape)
     if (!layoutStyle && bool(shape->m_style) && shape->m_style->has<property::LayoutStyle>())
       layoutStyle=shape->m_style->get<property::LayoutStyle>();
     fillLayoutProps(layoutStyle, styleProps);
+    fillTextAutoSizeProps(shape->m_resizeFlags,shape->m_geometry,styleProps);
     return drawTextBox(shape->m_text, trafo, shape->m_geometry, styleProps);
   }
 
@@ -1242,6 +1226,7 @@ void IWORKCollector::drawShape(const IWORKShapePtr_t &shape)
     if (!layoutStyle && bool(shape->m_style) && shape->m_style->has<property::LayoutStyle>())
       layoutStyle=shape->m_style->get<property::LayoutStyle>();
     fillLayoutProps(layoutStyle, props);
+    fillTextAutoSizeProps(shape->m_resizeFlags,shape->m_geometry,props);
     drawTextBox(shape->m_text, trafo, shape->m_geometry, props);
   }
 }
