@@ -7,6 +7,8 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  */
 
+#include "VSDParser.h"
+
 #include <librevenge-stream/librevenge-stream.h>
 #include <locale.h>
 #include <cassert>
@@ -15,7 +17,6 @@
 #include <cmath>
 #include <set>
 #include "libvisio_utils.h"
-#include "VSDParser.h"
 #include "VSDInternalStream.h"
 #include "VSDDocumentStructure.h"
 #include "VSDContentCollector.h"
@@ -41,7 +42,7 @@ void libvisio::VSDParser::_nameFromId(VSDName &name, unsigned id, unsigned level
   std::map<unsigned, std::map<unsigned, VSDName> >::const_iterator iter1 = m_namesMapMap.find(level);
   if (iter1 != m_namesMapMap.end())
   {
-    std::map<unsigned, VSDName>::const_iterator iter = iter1->second.find(id);
+    auto iter = iter1->second.find(id);
     if (iter != iter1->second.end())
       name = iter->second;
   }
@@ -343,7 +344,7 @@ void libvisio::VSDParser::handleStream(const Pointer &ptr, unsigned idx, unsigne
     break;
   case VSD_OLE_LIST:
     if (!m_shape.m_foreign)
-      m_shape.m_foreign = new ForeignData();
+      m_shape.m_foreign = make_unique<ForeignData>();
     m_shape.m_foreign->dataId = idx;
     break;
   default:
@@ -770,7 +771,7 @@ void libvisio::VSDParser::readForeignData(librevenge::RVNGInputStream *input)
   librevenge::RVNGBinaryData binaryData(buffer, tmpBytesRead);
 
   if (!m_shape.m_foreign)
-    m_shape.m_foreign = new ForeignData();
+    m_shape.m_foreign = make_unique<ForeignData>();
   m_shape.m_foreign->dataId = m_header.id;
   m_shape.m_foreign->data = binaryData;
 }
@@ -788,7 +789,7 @@ void libvisio::VSDParser::readOLEData(librevenge::RVNGInputStream *input)
   librevenge::RVNGBinaryData oleData(buffer, tmpBytesRead);
 
   if (!m_shape.m_foreign)
-    m_shape.m_foreign = new ForeignData();
+    m_shape.m_foreign = make_unique<ForeignData>();
   // Append data instead of setting it - allows multi-stream OLE objects
   m_shape.m_foreign->data.append(oleData);
 
@@ -938,7 +939,7 @@ void libvisio::VSDParser::readGeomList(librevenge::RVNGInputStream *input)
     if (childrenListLength > getRemainingLength(input))
       childrenListLength = getRemainingLength(input);
     geometryOrder.reserve(childrenListLength / sizeof(uint32_t));
-    for (unsigned i = 0; i < (childrenListLength / sizeof(uint32_t)); i++)
+    for (size_t i = 0; i < (childrenListLength / sizeof(uint32_t)); i++)
       geometryOrder.push_back(readU32(input));
 
     if (m_currentGeometryList)
@@ -965,7 +966,7 @@ void libvisio::VSDParser::readCharList(librevenge::RVNGInputStream *input)
       childrenListLength = getRemainingLength(input);
     std::vector<unsigned> characterOrder;
     characterOrder.reserve(childrenListLength / sizeof(uint32_t));
-    for (unsigned i = 0; i < (childrenListLength / sizeof(uint32_t)); i++)
+    for (size_t i = 0; i < (childrenListLength / sizeof(uint32_t)); i++)
       characterOrder.push_back(readU32(input));
 
     m_shape.m_charList.setElementsOrder(characterOrder);
@@ -987,7 +988,7 @@ void libvisio::VSDParser::readParaList(librevenge::RVNGInputStream *input)
       childrenListLength = getRemainingLength(input);
     std::vector<unsigned> paragraphOrder;
     paragraphOrder.reserve(childrenListLength / sizeof(uint32_t));
-    for (unsigned i = 0; i < (childrenListLength / sizeof(uint32_t)); i++)
+    for (size_t i = 0; i < (childrenListLength / sizeof(uint32_t)); i++)
       paragraphOrder.push_back(readU32(input));
 
     m_shape.m_paraList.setElementsOrder(paragraphOrder);
@@ -1013,7 +1014,7 @@ void libvisio::VSDParser::readTabsDataList(librevenge::RVNGInputStream *input)
       childrenListLength = getRemainingLength(input);
     std::vector<unsigned> tabsOrder;
     tabsOrder.reserve(childrenListLength / sizeof(uint32_t));
-    for (unsigned i = 0; i < (childrenListLength / sizeof(uint32_t)); i++)
+    for (size_t i = 0; i < (childrenListLength / sizeof(uint32_t)); i++)
       tabsOrder.push_back(readU32(input));
   }
 }
@@ -1033,7 +1034,7 @@ void libvisio::VSDParser::readLayerList(librevenge::RVNGInputStream *input)
       childrenListLength = getRemainingLength(input);
     std::vector<unsigned> layerOrder;
     layerOrder.reserve(childrenListLength / sizeof(uint32_t));
-    for (unsigned i = 0; i < (childrenListLength / sizeof(uint32_t)); i++)
+    for (size_t i = 0; i < (childrenListLength / sizeof(uint32_t)); i++)
       layerOrder.push_back(readU32(input));
   }
 }
@@ -1154,7 +1155,7 @@ void libvisio::VSDParser::readXFormData(librevenge::RVNGInputStream *input)
 void libvisio::VSDParser::readXForm1D(librevenge::RVNGInputStream *input)
 {
   if (!m_shape.m_xform1d)
-    m_shape.m_xform1d = new XForm1D();
+    m_shape.m_xform1d = make_unique<XForm1D>();
   input->seek(1, librevenge::RVNG_SEEK_CUR);
   m_shape.m_xform1d->beginX = readDouble(input);
   input->seek(1, librevenge::RVNG_SEEK_CUR);
@@ -1167,9 +1168,7 @@ void libvisio::VSDParser::readXForm1D(librevenge::RVNGInputStream *input)
 
 void libvisio::VSDParser::readTxtXForm(librevenge::RVNGInputStream *input)
 {
-  if (m_shape.m_txtxform)
-    delete m_shape.m_txtxform;
-  m_shape.m_txtxform = new XForm();
+  m_shape.m_txtxform = make_unique<XForm>();
   input->seek(1, librevenge::RVNG_SEEK_CUR);
   m_shape.m_txtxform->pinX = readDouble(input);
   input->seek(1, librevenge::RVNG_SEEK_CUR);
@@ -1208,7 +1207,7 @@ void libvisio::VSDParser::readShapeList(librevenge::RVNGInputStream *input)
       childrenListLength = getRemainingLength(input);
     std::vector<unsigned> shapeOrder;
     shapeOrder.reserve(childrenListLength / sizeof(uint32_t));
-    for (unsigned i = 0; i < (childrenListLength / sizeof(uint32_t)); i++)
+    for (size_t i = 0; i < (childrenListLength / sizeof(uint32_t)); i++)
       shapeOrder.push_back(readU32(input));
 
     if (!m_isShapeStarted)
@@ -1236,7 +1235,7 @@ void libvisio::VSDParser::readForeignDataType(librevenge::RVNGInputStream *input
   unsigned foreignFormat = readU32(input);
 
   if (!m_shape.m_foreign)
-    m_shape.m_foreign = new ForeignData();
+    m_shape.m_foreign = make_unique<ForeignData>();
   m_shape.m_foreign->typeId = m_header.id;
   m_shape.m_foreign->type = foreignType;
   m_shape.m_foreign->format = foreignFormat;
@@ -1283,11 +1282,11 @@ void libvisio::VSDParser::readShape(librevenge::RVNGInputStream *input)
     m_currentShapeID = m_header.id;
   m_currentShapeLevel = m_header.level;
   unsigned parent = 0;
-  unsigned masterPage = MINUS_ONE;
-  unsigned masterShape = MINUS_ONE;
-  unsigned lineStyle = MINUS_ONE;
-  unsigned fillStyle = MINUS_ONE;
-  unsigned textStyle = MINUS_ONE;
+  auto masterPage = MINUS_ONE;
+  auto masterShape = MINUS_ONE;
+  auto lineStyle = MINUS_ONE;
+  auto fillStyle = MINUS_ONE;
+  auto textStyle = MINUS_ONE;
 
   try
   {
@@ -1314,10 +1313,10 @@ void libvisio::VSDParser::readShape(librevenge::RVNGInputStream *input)
   if (tmpShape)
   {
     if (tmpShape->m_foreign)
-      m_shape.m_foreign = new ForeignData(*(tmpShape->m_foreign));
+      m_shape.m_foreign = make_unique<ForeignData>(*(tmpShape->m_foreign));
     m_shape.m_xform = tmpShape->m_xform;
     if (tmpShape->m_txtxform)
-      m_shape.m_txtxform = new XForm(*(tmpShape->m_txtxform));
+      m_shape.m_txtxform = make_unique<XForm>(*(tmpShape->m_txtxform));
     m_shape.m_tabSets = tmpShape->m_tabSets;
     m_shape.m_text = tmpShape->m_text;
     m_shape.m_textFormat = tmpShape->m_textFormat;
@@ -1723,7 +1722,7 @@ void libvisio::VSDParser::readFieldList(librevenge::RVNGInputStream *input)
       childrenListLength = getRemainingLength(input);
     std::vector<unsigned> fieldOrder;
     fieldOrder.reserve(childrenListLength / sizeof(uint32_t));
-    for (unsigned i = 0; i < (childrenListLength / sizeof(uint32_t)); i++)
+    for (size_t i = 0; i < (childrenListLength / sizeof(uint32_t)); i++)
       fieldOrder.push_back(readU32(input));
 
     m_shape.m_fields.setElementsOrder(fieldOrder);
@@ -1771,14 +1770,14 @@ void libvisio::VSDParser::readFontIX(librevenge::RVNGInputStream *input)
 {
   long tmpAdjust = input->tell();
   input->seek(2, librevenge::RVNG_SEEK_CUR);
-  unsigned char codePage = (unsigned char)(getUInt(input) & 0xff);
+  auto codePage = (unsigned char)(getUInt(input) & 0xff);
   tmpAdjust -= input->tell();
 
   std::string fontName;
 
   for (long i = 0; i < (long)(m_header.dataLength + tmpAdjust); i++)
   {
-    char curchar = (char)readU8(input);
+    auto curchar = (char)readU8(input);
     if (curchar == 0)
       break;
     fontName.append(1, curchar);
@@ -2173,8 +2172,8 @@ void libvisio::VSDParser::readTextField(librevenge::RVNGInputStream *input)
 {
   unsigned long initialPosition = input->tell();
   input->seek(7, librevenge::RVNG_SEEK_CUR);
-  unsigned char tmpCode = readU8(input);
-  if (tmpCode == 0xe8)
+  unsigned char cellType = readU8(input);
+  if (cellType == CELL_TYPE_StringWithoutUnit)
   {
     int nameId = readS32(input);
     input->seek(6, librevenge::RVNG_SEEK_CUR);
@@ -2228,13 +2227,13 @@ void libvisio::VSDParser::readTextField(librevenge::RVNGInputStream *input)
 
     if (blockIdx != 2)
     {
-      if (tmpCode == 0x28)
-        formatNumber = 200;
+      if (cellType == CELL_TYPE_Date)
+        formatNumber = VSD_FIELD_FORMAT_MsoDateShort;
       else
-        formatNumber = 0xffff;
+        formatNumber = VSD_FIELD_FORMAT_Unknown;
     }
 
-    m_shape.m_fields.addNumericField(m_header.id, m_header.level, formatNumber, numericValue, formatStringId);
+    m_shape.m_fields.addNumericField(m_header.id, m_header.level, formatNumber, cellType, numericValue, formatStringId);
   }
 }
 
@@ -2268,7 +2267,7 @@ void libvisio::VSDParser::readMisc(librevenge::RVNGInputStream *input)
             if (0x40000073 == readU32(input))
             {
               if (!m_shape.m_xform1d)
-                m_shape.m_xform1d = new XForm1D();
+                m_shape.m_xform1d = make_unique<XForm1D>();
               if (m_shape.m_xform1d->beginId == MINUS_ONE)
                 m_shape.m_xform1d->beginId = shapeId;
               else if (m_shape.m_xform1d->endId == MINUS_ONE)
