@@ -1,13 +1,19 @@
 /*
  * XML Security Library (http://www.aleksey.com/xmlsec).
  *
- * XML Parser transform and utility functions.
  *
  * This is free software; see Copyright file in the source
  * distribution for preciese wording.
  *
  * Copyright (C) 2002-2016 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
  */
+/**
+ * SECTION:parser
+ * @Short_description: XML parser functions and the XML parser transform implementation.
+ * @Stability: Stable
+ *
+ */
+
 #include "globals.h"
 
 #include <stdlib.h>
@@ -126,6 +132,10 @@ xmlSecParserFinalize(xmlSecTransformPtr transform) {
     xmlSecAssert(ctx != NULL);
 
     if(ctx->parserCtx != NULL) {
+        if(ctx->parserCtx->myDoc != NULL) {
+            xmlFreeDoc(ctx->parserCtx->myDoc);
+	    ctx->parserCtx->myDoc = NULL;
+        }
         xmlFreeParserCtxt(ctx->parserCtx);
     }
     memset(ctx, 0, sizeof(xmlSecParserCtx));
@@ -133,7 +143,7 @@ xmlSecParserFinalize(xmlSecTransformPtr transform) {
 
 static int
 xmlSecParserPushBin(xmlSecTransformPtr transform, const xmlSecByte* data,
-                                xmlSecSize dataSize, int final, xmlSecTransformCtxPtr transformCtx) {
+                    xmlSecSize dataSize, int final, xmlSecTransformCtxPtr transformCtx) {
     xmlSecParserCtxPtr ctx;
     int ret;
 
@@ -152,11 +162,7 @@ xmlSecParserPushBin(xmlSecTransformPtr transform, const xmlSecByte* data,
             xmlSecXmlError("xmlCreatePushParserCtxt", xmlSecTransformGetName(transform));
             return(-1);
         }
-
-        /* required for c14n! */
-        ctx->parserCtx->loadsubset      = XML_DETECT_IDS | XML_COMPLETE_ATTRS;
-        ctx->parserCtx->replaceEntities = 1;
-        ctx->parserCtx->options         = XML_PARSE_NONET;
+        xmlSecParsePrepareCtxt(ctx->parserCtx);
 
         transform->status = xmlSecTransformStatusWorking;
     } else if(transform->status == xmlSecTransformStatusFinished) {
@@ -273,6 +279,7 @@ xmlSecParserPopXml(xmlSecTransformPtr transform, xmlSecNodeSetPtr* nodes,
         xmlFreeParserInputBuffer(buf);
         return(-1);
     }
+    xmlSecParsePrepareCtxt(ctxt);
 
     input = xmlNewIOInputStream(ctxt, buf, XML_CHAR_ENCODING_NONE);
     if(input == NULL) {
@@ -288,14 +295,14 @@ xmlSecParserPopXml(xmlSecTransformPtr transform, xmlSecNodeSetPtr* nodes,
         xmlSecXmlParserError("inputPush", ctxt,
                              xmlSecTransformGetName(transform));
         xmlFreeInputStream(input);
+        if(ctxt->myDoc != NULL) {
+            xmlFreeDoc(ctxt->myDoc);
+            ctxt->myDoc = NULL;
+        }
         xmlFreeParserCtxt(ctxt);
         return(-1);
     }
 
-    /* required for c14n! */
-    ctxt->loadsubset = XML_DETECT_IDS | XML_COMPLETE_ATTRS;
-    ctxt->replaceEntities = 1;
-
     /* finaly do the parsing */
     ret = xmlParseDocument(ctxt);
     if(ret < 0) {
@@ -366,9 +373,7 @@ xmlSecParseFile(const char *filename) {
                         "filename=%s", xmlSecErrorsSafeString(filename));
         return(NULL);
     }
-
-    /* enable parsing of XML documents with large text nodes */
-    /* crashes on x64 xmlCtxtUseOptions (ctxt, XML_PARSE_HUGE); */
+    xmlSecParsePrepareCtxt(ctxt);
 
     /* todo: set directories from current doc? */
     if ((ctxt->directory == NULL) && (directory == NULL)) {
@@ -389,29 +394,32 @@ xmlSecParseFile(const char *filename) {
         }
     }
 
-    /* required for c14n! */
-    ctxt->loadsubset = XML_DETECT_IDS | XML_COMPLETE_ATTRS;
-    ctxt->replaceEntities = 1;
-
     ret = xmlParseDocument(ctxt);
     if(ret < 0) {
         xmlSecXmlParserError2("xmlParseDocument", ctxt, NULL,
                               "filename=%s",
                               xmlSecErrorsSafeString(filename));
+        if(ctxt->myDoc != NULL) {
+            xmlFreeDoc(ctxt->myDoc);
+            ctxt->myDoc = NULL;
+        }
         xmlFreeParserCtxt(ctxt);
         return(NULL);
     }
 
     if(!ctxt->wellFormed) {
        xmlSecInternalError("document is not well formed", NULL);
-       xmlFreeDoc(ctxt->myDoc);
-       ctxt->myDoc = NULL;
+       if(ctxt->myDoc != NULL) {
+           xmlFreeDoc(ctxt->myDoc);
+           ctxt->myDoc = NULL;
+       }
        xmlFreeParserCtxt(ctxt);
        return(NULL);
     }
 
     /* done */
     res = ctxt->myDoc;
+    ctxt->myDoc = NULL;
     xmlFreeParserCtxt(ctxt);
     return(res);
 
@@ -444,10 +452,7 @@ xmlSecParseMemoryExt(const xmlSecByte *prefix, xmlSecSize prefixSize,
         xmlSecXmlError("xmlCreatePushParserCtxt", NULL);
         goto done;
     }
-
-    /* required for c14n! */
-    ctxt->loadsubset = XML_DETECT_IDS | XML_COMPLETE_ATTRS;
-    ctxt->replaceEntities = 1;
+    xmlSecParsePrepareCtxt(ctxt);
 
     /* prefix */
     if((prefix != NULL) && (prefixSize > 0)) {
@@ -489,9 +494,14 @@ xmlSecParseMemoryExt(const xmlSecByte *prefix, xmlSecSize prefixSize,
         goto done;
     }
     doc = ctxt->myDoc;
+    ctxt->myDoc = NULL;
 
 done:
     if(ctxt != NULL) {
+        if(ctxt->myDoc != NULL) {
+            xmlFreeDoc(ctxt->myDoc);
+            ctxt->myDoc = NULL;
+        }
         xmlFreeParserCtxt(ctxt);
     }
     return(doc);
@@ -522,29 +532,56 @@ xmlSecParseMemory(const xmlSecByte *buffer, xmlSecSize size, int recovery) {
         xmlSecXmlError("xmlCreateMemoryParserCtxt", NULL);
         return(NULL);
     }
-
-    /* required for c14n! */
-    ctxt->loadsubset = XML_DETECT_IDS | XML_COMPLETE_ATTRS;
-    ctxt->replaceEntities = 1;
+    xmlSecParsePrepareCtxt(ctxt);
 
     ret = xmlParseDocument(ctxt);
     if(ret < 0) {
         xmlSecXmlParserError("xmlParseDocument", ctxt, NULL);
+        if(ctxt->myDoc != NULL) {
+            xmlFreeDoc(ctxt->myDoc);
+            ctxt->myDoc = NULL;
+        }	
         xmlFreeParserCtxt(ctxt);
         return(NULL);
     }
 
     if(!(ctxt->wellFormed) && !recovery) {
         xmlSecInternalError("document is not well formed", NULL);
-        xmlFreeDoc(ctxt->myDoc);
-        ctxt->myDoc = NULL;
+        if(ctxt->myDoc != NULL) {
+            xmlFreeDoc(ctxt->myDoc);
+            ctxt->myDoc = NULL;
+        }
         xmlFreeParserCtxt(ctxt);
         return(NULL);
     }
 
     /* done */
     res = ctxt->myDoc;
+    ctxt->myDoc = NULL;
     xmlFreeParserCtxt(ctxt);
     return(res);
 }
 
+/**
+ * xmlSecParsePrepareCtxt:
+ * @ctxt:               the parser context
+ *
+ * Prepares parser context for parsing XML for XMLSec.
+ */
+void
+xmlSecParsePrepareCtxt(xmlParserCtxtPtr ctxt) {
+	xmlSecAssert(ctxt != NULL);
+
+    /* required for c14n! */
+    ctxt->loadsubset = XML_DETECT_IDS | XML_COMPLETE_ATTRS;
+    ctxt->replaceEntities = 1;
+
+    /*
+     * Also see xmlSecReplaceNodeBufferAndReturn:
+     *
+     * XML_PARSE_NONET  to support c14n
+     * XML_PARSE_NODICT to avoid problems with moving nodes around
+     * XML_PARSE_HUGE   to enable parsing of XML documents with large text nodes
+     */
+    xmlCtxtUseOptions(ctxt, XML_PARSE_NONET | XML_PARSE_NODICT | XML_PARSE_HUGE);
+}
