@@ -5,12 +5,13 @@
  * found in the LICENSE file.
  */
 
+#include "include/core/SkColorSpace.h"
 #include "include/core/SkSurface.h"
 #include "include/gpu/GrDirectContext.h"
-#include "src/gpu/GrDirectContextPriv.h"
-#include "src/gpu/gl/GrGLDefines.h"
-#include "src/gpu/gl/GrGLGpu.h"
-#include "src/gpu/gl/GrGLUtil.h"
+#include "src/gpu/ganesh/GrDirectContextPriv.h"
+#include "src/gpu/ganesh/gl/GrGLDefines_impl.h"
+#include "src/gpu/ganesh/gl/GrGLGpu.h"
+#include "src/gpu/ganesh/gl/GrGLUtil.h"
 #include "tests/Test.h"
 
 #ifdef SK_GL
@@ -77,8 +78,15 @@ DEF_GPUTEST_FOR_GL_RENDERING_CONTEXTS(TextureBindingsResetTest, reporter, ctxInf
     static constexpr SkISize kDims = {10, 10};
     GrBackendFormat format = gpu->caps()->getDefaultBackendFormat(GrColorType::kRGBA_8888,
                                                                   GrRenderable::kNo);
-    auto tex = gpu->createTexture(kDims, format, GrTextureType::k2D, GrRenderable::kNo, 1,
-                                  GrMipmapped::kNo, SkBudgeted::kNo, GrProtected::kNo);
+    auto tex = gpu->createTexture(kDims,
+                                  format,
+                                  GrTextureType::k2D,
+                                  GrRenderable::kNo,
+                                  1,
+                                  GrMipmapped::kNo,
+                                  SkBudgeted::kNo,
+                                  GrProtected::kNo,
+                                  /*label=*/{});
     REPORTER_ASSERT(reporter, tex);
     dContext->resetGLTextureBindings();
     checkBindings();
