@@ -6,26 +6,26 @@
 
 #include "xfa/fxfa/layout/cxfa_contentlayoutitem.h"
 
-#include <utility>
-
 #include "fxjs/xfa/cjx_object.h"
+#include "third_party/base/check_op.h"
 #include "xfa/fxfa/cxfa_ffwidget.h"
 #include "xfa/fxfa/parser/cxfa_margin.h"
 #include "xfa/fxfa/parser/cxfa_node.h"
 
-CXFA_ContentLayoutItem::CXFA_ContentLayoutItem(
-    CXFA_Node* pNode,
-    std::unique_ptr<CXFA_FFWidget> pWidget)
-    : CXFA_LayoutItem(pNode, kContentItem), m_pFFWidget(std::move(pWidget)) {
+CXFA_ContentLayoutItem::CXFA_ContentLayoutItem(CXFA_Node* pNode,
+                                               CXFA_FFWidget* pWidget)
+    : CXFA_LayoutItem(pNode, kContentItem), m_pFFWidget(pWidget) {
   if (m_pFFWidget)
     m_pFFWidget->SetLayoutItem(this);
 }
 
-CXFA_ContentLayoutItem::~CXFA_ContentLayoutItem() {
-  if (m_pFFWidget)
-    m_pFFWidget->SetLayoutItem(nullptr);
+CXFA_ContentLayoutItem::~CXFA_ContentLayoutItem() = default;
 
-  RemoveSelf();
+void CXFA_ContentLayoutItem::Trace(cppgc::Visitor* visitor) const {
+  CXFA_LayoutItem::Trace(visitor);
+  visitor->Trace(m_pPrev);
+  visitor->Trace(m_pNext);
+  visitor->Trace(m_pFFWidget);
 }
 
 CXFA_ContentLayoutItem* CXFA_ContentLayoutItem::GetFirst() {
@@ -61,11 +61,13 @@ void CXFA_ContentLayoutItem::RemoveSelf() {
     m_pPrev->m_pNext = m_pNext;
 }
 
-CFX_RectF CXFA_ContentLayoutItem::GetRect(bool bRelative) const {
+CFX_RectF CXFA_ContentLayoutItem::GetRelativeRect() const {
+  return CFX_RectF(m_sPos, m_sSize);
+}
+
+CFX_RectF CXFA_ContentLayoutItem::GetAbsoluteRect() const {
   CFX_PointF sPos = m_sPos;
   CFX_SizeF sSize = m_sSize;
-  if (bRelative)
-    return CFX_RectF(sPos, sSize);
 
   for (CXFA_LayoutItem* pLayoutItem = GetParent(); pLayoutItem;
        pLayoutItem = pLayoutItem->GetParent()) {
