@@ -7,16 +7,16 @@
 #include "xfa/fxfa/parser/cxfa_acrobat.h"
 
 #include "fxjs/xfa/cjx_node.h"
-#include "third_party/base/ptr_util.h"
+#include "xfa/fxfa/parser/cxfa_document.h"
 
 namespace {
 
 const CXFA_Node::PropertyData kAcrobatPropertyData[] = {
-    {XFA_Element::AutoSave, 1, 0},
-    {XFA_Element::Validate, 1, 0},
-    {XFA_Element::ValidateApprovalSignatures, 1, 0},
-    {XFA_Element::Acrobat7, 1, 0},
-    {XFA_Element::Common, 1, 0},
+    {XFA_Element::AutoSave, 1, {}},
+    {XFA_Element::Validate, 1, {}},
+    {XFA_Element::ValidateApprovalSignatures, 1, {}},
+    {XFA_Element::Acrobat7, 1, {}},
+    {XFA_Element::Common, 1, {}},
 };
 
 const CXFA_Node::AttributeData kAcrobatAttributeData[] = {
@@ -29,11 +29,13 @@ const CXFA_Node::AttributeData kAcrobatAttributeData[] = {
 CXFA_Acrobat::CXFA_Acrobat(CXFA_Document* doc, XFA_PacketType packet)
     : CXFA_Node(doc,
                 packet,
-                XFA_XDPPACKET_Config,
+                XFA_XDPPACKET::kConfig,
                 XFA_ObjectType::Node,
                 XFA_Element::Acrobat,
                 kAcrobatPropertyData,
                 kAcrobatAttributeData,
-                pdfium::MakeUnique<CJX_Node>(this)) {}
+                cppgc::MakeGarbageCollected<CJX_Node>(
+                    doc->GetHeap()->GetAllocationHandle(),
+                    this)) {}
 
 CXFA_Acrobat::~CXFA_Acrobat() = default;
