@@ -11,8 +11,11 @@
 
 class CXFA_PaginationOverride final : public CXFA_Node {
  public:
-  CXFA_PaginationOverride(CXFA_Document* doc, XFA_PacketType packet);
+  CONSTRUCT_VIA_MAKE_GARBAGE_COLLECTED;
   ~CXFA_PaginationOverride() override;
+
+ private:
+  CXFA_PaginationOverride(CXFA_Document* doc, XFA_PacketType packet);
 };
 
 #endif  // XFA_FXFA_PARSER_CXFA_PAGINATIONOVERRIDE_H_
