@@ -7,15 +7,15 @@
 #include "xfa/fxfa/parser/cxfa_pcl.h"
 
 #include "fxjs/xfa/cjx_node.h"
-#include "third_party/base/ptr_util.h"
+#include "xfa/fxfa/parser/cxfa_document.h"
 
 namespace {
 
 const CXFA_Node::PropertyData kPclPropertyData[] = {
-    {XFA_Element::FontInfo, 1, 0},   {XFA_Element::Jog, 1, 0},
-    {XFA_Element::Xdc, 1, 0},        {XFA_Element::BatchOutput, 1, 0},
-    {XFA_Element::PageOffset, 1, 0}, {XFA_Element::OutputBin, 1, 0},
-    {XFA_Element::Staple, 1, 0},     {XFA_Element::MediumInfo, 1, 0},
+    {XFA_Element::FontInfo, 1, {}},   {XFA_Element::Jog, 1, {}},
+    {XFA_Element::Xdc, 1, {}},        {XFA_Element::BatchOutput, 1, {}},
+    {XFA_Element::PageOffset, 1, {}}, {XFA_Element::OutputBin, 1, {}},
+    {XFA_Element::Staple, 1, {}},     {XFA_Element::MediumInfo, 1, {}},
 };
 
 const CXFA_Node::AttributeData kPclAttributeData[] = {
@@ -29,11 +29,13 @@ const CXFA_Node::AttributeData kPclAttributeData[] = {
 CXFA_Pcl::CXFA_Pcl(CXFA_Document* doc, XFA_PacketType packet)
     : CXFA_Node(doc,
                 packet,
-                XFA_XDPPACKET_Config,
+                XFA_XDPPACKET::kConfig,
                 XFA_ObjectType::Node,
                 XFA_Element::Pcl,
                 kPclPropertyData,
                 kPclAttributeData,
-                pdfium::MakeUnique<CJX_Node>(this)) {}
+                cppgc::MakeGarbageCollected<CJX_Node>(
+                    doc->GetHeap()->GetAllocationHandle(),
+                    this)) {}
 
 CXFA_Pcl::~CXFA_Pcl() = default;
