@@ -11,8 +11,11 @@
 
 class CXFA_Picture final : public CXFA_Node {
  public:
-  CXFA_Picture(CXFA_Document* doc, XFA_PacketType packet);
+  CONSTRUCT_VIA_MAKE_GARBAGE_COLLECTED;
   ~CXFA_Picture() override;
+
+ private:
+  CXFA_Picture(CXFA_Document* doc, XFA_PacketType packet);
 };
 
 #endif  // XFA_FXFA_PARSER_CXFA_PICTURE_H_
