@@ -7,7 +7,7 @@
 #include "xfa/fxfa/parser/cxfa_reason.h"
 
 #include "fxjs/xfa/cjx_textnode.h"
-#include "third_party/base/ptr_util.h"
+#include "xfa/fxfa/parser/cxfa_document.h"
 
 namespace {
 
@@ -23,11 +23,13 @@ const CXFA_Node::AttributeData kReasonAttributeData[] = {
 CXFA_Reason::CXFA_Reason(CXFA_Document* doc, XFA_PacketType packet)
     : CXFA_Node(doc,
                 packet,
-                (XFA_XDPPACKET_Template | XFA_XDPPACKET_Form),
+                {XFA_XDPPACKET::kTemplate, XFA_XDPPACKET::kForm},
                 XFA_ObjectType::TextNode,
                 XFA_Element::Reason,
                 {},
                 kReasonAttributeData,
-                pdfium::MakeUnique<CJX_TextNode>(this)) {}
+                cppgc::MakeGarbageCollected<CJX_TextNode>(
+                    doc->GetHeap()->GetAllocationHandle(),
+                    this)) {}
 
 CXFA_Reason::~CXFA_Reason() = default;
