@@ -11,8 +11,11 @@
 
 class CXFA_Relevant final : public CXFA_Node {
  public:
-  CXFA_Relevant(CXFA_Document* doc, XFA_PacketType packet);
+  CONSTRUCT_VIA_MAKE_GARBAGE_COLLECTED;
   ~CXFA_Relevant() override;
+
+ private:
+  CXFA_Relevant(CXFA_Document* doc, XFA_PacketType packet);
 };
 
 #endif  // XFA_FXFA_PARSER_CXFA_RELEVANT_H_
